%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                Geometry.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Description: This function deals with the geometry of the buoy, first the
%waterline is found. Then the buoy is divided into panels and plotted.
%Finally the panel midpoint coordinates and panel areas are calculated. 
%The purpose is to have a basis for a simple panel method to calculate the 
%forces and motions on the buoy. It also serves as a input validation for 
%the user.
%
%Called by: main.m
%
%Calls: Nothing
%
%Outputs: data
%
%Version: 1.0
%Written by: Jon Marius Aasheim, 2012
%This code was inspired by a similar function in Glomnes and Viko
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = Geometry(data)
%%
%Plotting the buoy geometry for buoy alternative 1
%
x = data.body.x;
z = data.body.z;

sectors= data.body.Nsec;

%%
% Finding waterline crossing for buoy alternative 1
%
if isempty(find(z==0)) %#ok
    for i = 1:length(x)-1;
        try                             
            if z(i) < 0                 %This checks if the buoy vertical component is below the waterline
                if z(i+1) > 0           %This checks if the next buoy vertical component is above the waterline
                    xZero = x(i) + ...
                        ((x(i+1)-x(i))/(z(i+1)-z(i)))* abs(z(i));  %Finds the x-value at waterline crossing
                    x = [x(find(z<0)) xZero x(find(z>0))];%#ok     %places xZero between x values for negative z and positive z
                    z = [z(find(z<0)) 0 z(find(z>0))];%#ok         %%places 0 between x values for negative z and positive z
                    break
                end
            end   
        catch                           %This command runs if the commands within the try-loop fails, and it will stop the program as the buoy does not have a waterline.
            disp('')
            disp('              ERROR              ')
            disp('Buoy does not break the waterline')
            disp('')
            break
        end
    end
end
data.body.x=x;
data.body.z=z;

X1=zeros(1,sectors+1);                         %a line of zeroes to create coordinate matrix
Y1=zeros(1,sectors+1); 
Z1=zeros(1,sectors+1);

numSegments = 0;


figure(1)
hold on                                 %All plots in the same figure until "hold off"
for i = 1:length(x)-1                   
    
    dz = (z(i+1)-z(i));                 %Making segment height in z direction
    dx = x(i):(x(i+1)-x(i))/15:x(i+1);  %Making segment length in x direction, this changes with each i
    
    if isempty(dx)
        dx = x(i)*ones(1,length(x)); %when sides are vertical a spesial segmentation is needed.
    end
    
    [X,Y,Z] = cylinder(dx,(sectors));          %Creates a uniform cylinder based on the vector contained in dx
    
    Z = Z .* dz + z(i);                 %Assigns the correct vertical coordinate for the cylinder piece 

    X1=[X1;X];                           %Adds all x coordinates to a large matrix for later use
    Y1=[Y1;Y];                          %Adds all y coordinates to a large matrix for later use
    Z1=[Z1;Z];                           %Adds all z coordinates to a large matrix for later use
    
    
    numSegments = numSegments +length(dx); %To get the actual number of element tiers in the model
    
    surf(X,Y,Z,'edgecolor',[0.2 0.2 0.8],'facecolor',[0.3 0.3 1]) %Creates a part in figure 1
    
end
hold off



set(gca,'CameraPosition', [max(x)*3 max(x)*3 max(x)*2])
axis equal  %% first 'axis equal' doen't seam to take
axis equal  % Shows both axis with the same values
%pause(3);


for i=2:(numSegments) %first line of placeholder values are ignored.
    dZ=Z1(i+1,1)-Z1(i,1);
    dX=X1(i+1,1)-X1(i,1);
    dY=((Y1(i+1,2)-Y1(i+1,1))+(Y1(i,2)-Y1(i,1)))/2;
    alpha=atan(dZ/dX);
    for j=1:(sectors/2+1)      %Due to symmetry only half of the bouy panels are saved
        
        
        

        %Finds the senter point of the element
        data.body.xmid(i-1,j) = X1(i,j) + (dX)/2;   
                
        data.body.ymid(i-1,j) = Y1(i,j) + (Y1(i+1,j)-Y1(i,j))/2; 
        
        data.body.zmid(i-1,j) = Z1(i,1) + (dZ)/2;
        
    end
    %z coordinate corresponding to x coordinate value, only one value is
    %stored per point as all x values at a given i have the same z-values.
    
    
    %Calculates the panel area
    if dZ==0 && alpha==0                %if paralell to seafloor
        data.body.PanelArea(i,1)=abs(dX*dY);

        
    elseif dZ~=0 && abs(alpha)==pi/2    %if perpendicular to seafloor
        data.body.PanelArea(i,1)=dZ*dY;
        
        
    elseif dZ==0 && dX==0               %if panel has no area
        data.body.PanelArea(i,1)=0;
        
    else                                %panel is slanted
        
        dS=sqrt((dZ)^2+(dX)^2);
        data.body.PanelArea(i,1)=abs(dS*dY);
        
    end   
    
end

