%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              IceCalc.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Description: This function calculates the static drift-off and heeling of
%the buoy due to level ice. 
%
%Called by: Analysis
%
%Calls: Restoring, sparseinv
%
%Outputs: Results
%
%Version: 1.0
%Written by: Nils Gunnar Viko and Einar B. Glomnes
%
%Version: 2.0
%Written by: Jon Marius Aasheim, 2012
%Changed calculation method for upwards slanting buoy. Commented, and
%changed the code to work for new program and make it more readable
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function Results = IceCalc(data, Results)


%body spcific input

x       =     data.body.x;                  
z       =     data.body.z;

KG      =     data.body.KG;
d       =     data.body.d;

%Enviroment specific input
g       =     data.env.g;                    
rho_w   =     data.env.rho;

%Ice specific input
rho_i   =     data.ice.rho;          %% Density of ice [kg/m^3]
sigmaf  =     data.ice.sigmaf;       %% Flexural strength of ice [Pa]
E       =     data.ice.E;            %% Young's modulus of ice [Pa]
nu      =     data.ice.nu;           %% Poisson's ratio for ice [-]
mu      =     data.ice.mu;           %% Buoy-ice friction coefficient [-]



%The buoy cannot be in motion and be subjected to ice. Might be added later

eta5 = 0;
eta3 = 0;
eta1 = 0;

h=0.5;

% Two alternative theories are applied depending on whether the slope of 
% the buoy is upwards or downwards.


%The buoy diameter is found in the waterline.
D          = 2*x(find( z == 0));


%The initial slope angle in the waterline is found. 
alpha0     =  atan(z(find(z == 0)-1)/(x(find(z == 0)) - x(find( z == 0)-1)));

MaxIce     =  data.ice.MaxIce;
MinIce     =  data.ice.MinIce;

dthickness = (MaxIce - MinIce)/50;
    
i=1;
for h = MinIce:dthickness:MaxIce
 
    iter=0; 
        

    if alpha0>=0 %slope is upwards in waterline, regular Croasdale is used
        
        if i==1
            eta5 = 0;
            eta3 = 0;
            eta1 = 0;

        else
            eta5 = r(5,1,i-1);
            eta3 = r(3,1,i-1);
            eta1 = r(1,1,i-1);
        end

        while 1
            
            %% Coefficients of Croasdale method
            alpha     =     alpha0 - eta5;

            ceta1     =     sin(alpha)+mu*cos(alpha);

            ceta2     =     cos(alpha)-mu*sin(alpha);
 
            C1        =     0.68*ceta1/ceta2;

            C2        =     ceta1*(ceta1/ceta2 + cot(alpha));
            
            %% Ice Rubble height
            hr        =     7.6*h^0.64;
            
            %% Forces and moments calculation,
            FH        =     C1*D*sigmaf*(((rho_w*g*h^5)/E)^0.25) ...
                            + C2*D*hr*h*rho_i*g;

            FV        =     0;
            M         =     -FH * (D/2 * sin(eta5) + d - KG); 
            
            %% Excitation vector and displacement vectors are defined   
            Fexc      =     [FH 0 FV 0 M 0]';

            rnow      =     zeros(6,3);
            
            rnow(1,1) =     eta1;
            rnow(3,1) =     eta3;
            rnow(5,1) =     eta5;
            
            %% Restoring force and Restoring coefficients are calculated
            C         =     Restoring(data,rnow,1,'no waves');
            Fr        =     Restoring(data,rnow,2,'no waves');
            
            %% An assesment of the error between restoring and excitation 

            e         =     sqrt(  ((Fr(1)+Fexc(1))/Fexc(1))^2 + ...
                            ((Fr(5)+Fexc(5))/Fexc(5))^2 );   
              
            %% Error is less than tolerance limit, ends while loop
            if e < 0.001

                r(:,1,i) = rnow(:,1);
                F(:,1,i) = Fexc;
                break;
            
            %% If 100 iteration has been made, ends while loop
            elseif iter == 100;
            
                disp('No Convergence')
                r(:,1,i) = rnow(:,1);
                F(:,1,i) = Fexc;
                break
            
            %% Calculates the next step and while loop continues
            else

                dr = sparseinv(C)*(Fexc+Fr);
                eta1 = eta1 + dr(1);
                eta3 = eta3 + dr(3);
                eta5 = eta5 + dr(5);
                iter = iter + 1;

            end



        end

    elseif alpha0<0 %slope is downwards, Timco-Cornett is used
        if i==1
            eta5 = 0;
            eta3 = 0;
            eta1 = 0;

        else
            eta5 = r(5,1,i-1);
            eta3 = r(3,1,i-1);
            eta1 = r(1,1,i-1);
        end


        while 1
            
            %% Coefficients of Timco-Cornett
            alpha      =    alpha0 + eta5;

            alphaTC    =    abs(alpha);

            ceta3      =    mu*cos(alphaTC) - sin(alphaTC);

            ceta4      =    mu*sin(alphaTC) + cos(alphaTC);

            C3         =    0.68*(ceta3/ceta4);

            C4         =    ceta3*((ceta3/ceta4)-cot(alphaTC));
            
            %% Forces and moments calculation,
            FH         =    C3*D*sigmaf*(((rho_w*g*h^5)/E)^0.25)...
                            + C4*D*d*h*(rho_w - rho_i)*g;

            FV         =    FH*(ceta3/ceta4);

            M          =    -FH * (D/2 * sin(eta5) + d - KG)... 
                            - FV * D/2 * cos(eta5);
            %% Excitation vector and displacement vectors are defined 
            Fexc       =    [FH 0 FV 0 M 0]';

            rnow       =    zeros(6,3);
            rnow(1,1)  =    eta1;
            rnow(3,1)  =    eta3;
            rnow(5,1)  =    eta5;

            %% Restoring force and Restoring coefficients are calculated
            C          =    Restoring(data,rnow,1,'no waves');
            Fr         =    Restoring(data,rnow,2,'no waves');

            e          =    sqrt(  ((Fr(1)+Fexc(1))/Fexc(1))^2 ...
                            + ((Fr(5)+Fexc(5))/Fexc(5))^2 );   
            
            %% Error is less than tolerance limit, ends while loop
            if e < 0.001

                r(:,1,i) = rnow(:,1);
                F(:,1,i) = Fexc;
                break;
            
            %% If 100 iteration has been made, ends while loop
            elseif iter == 100;

                disp('No Convergence')
                r(:,1,i) = rnow(:,1);
                F(:,1,i) = Fexc;
                break
            %% Calculates the next step and while loop continues
            else
            
                dr   = sparseinv(C)*(Fexc+Fr);
                eta1 = eta1 + dr(1);
                eta3 = eta3 + dr(3);
                eta5 = eta5 + dr(5);
                iter = iter + 1;

            end

        end

    else

       disp('Something has gone wrong with the geometry')


    end
i=i+1;
end
         
%% Returns the results      

Results.ice.r = r;

Results.ice.F = F;

