%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           MotionAnalysis.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Description: This function does all the calculation pertaining to the
%motions. It includes Force calculations, acceleration, velocity and
%displacement for a 3-degree of freedom system. (Surge, Heave and Pitch)
%The calculations are done incrementally for each time step in the
%following order. Pressure forces and moments are calculated, Restoring
%forces and coefficients are calculated, The sum of all forces are
%calulated, accelerations are calculated by Newtons second law, a
%Newmark-beta step integration is applied to find velocity and
%displacements. Finally the structure is moved according to the calculated
%displacements and the initial values are also updated. 
%
%Called by: Analysis
%
%Calls: Pressure, Restoring
%
%Outputs: Results
%
%Version: 1.0
%Written by: Jon Marius Aasheim, 2012
%
%Version:
%Written by: 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Results = MotionAnalysis(data, Run)

    a01         =   0;
    a03         =   0;
    a05         =   0;
    
    v01         =   0;
    v03         =   0;
    v05         =   0;
    
    %initial values to be used for restoring coefficient calculations
    
    s01         =   20;         
    s03         =   0.5;
    s05         =   0.5;
    
    ds1         =   0;
    ds3         =   0;
    ds5         =   0;
    
    dv5         =   0;
    At          =   Run.AnalysisTime;
    dt          =   Run.TimeStep;
    time        =   At/dt;

    ceta_a      =   data.env.WaveHeight/2;
    T           =   data.env.WavePeriod;
    omega       =   2*pi/T;
    
    gamma       =   0.5;
    beta        =   0.25;
    
    analysis.x1 =   data.body.xmid;
    analysis.z1 =   data.body.zmid;
    
    rnow(1,1)               =   s01;
    rnow(3,1)               =   s03; 
    rnow(5,1)               =   s05;
    
    ksi1        =   data.body.ksi1;          %% Damping ratio, surge [-]
    ksi3        =   data.body.ksi3;          %% Damping ratio, heave [-]
    ksi5        =   data.body.ksi5;          %% Damping ratio, pitch [-]
    
    [RestCoeff]  =   Restoring(data,rnow,1,0);

    
for t=1:time
    
    data.t=t*dt;
    
    %% Calculating the pressure forces and moments   
    
    data=Pressure(data, analysis);
   
    %% Calculating the Restoring forces and moments
    
    rnow(1,1)               =   s01;
    rnow(3,1)               =   s03; 
    rnow(5,1)               =   s05;
    
  
    
    %% Transfering the pressure forces and multiplicatio with factor 2 and 4 due to symmetry
    
    %symmetry around one axis
    Results.VertStatic(t)   =   2*data.VertStatic;
    
    %symmetry around two axis
    Results.VertDynamic(t)  =   4*data.VertDynamic;
    
    Results.HorStatic(t)    =   0;
    Results.HorDynamic(t)   =   4*data.HorDynamic;

    Results.MomStatic(t)    =   0;
    Results.MomDynamic(t)   =   4*data.MomDynamic;       

    
    %% Calculating the total force on the structure
    
    Results.HorTotal(t)     =   ...
    Results.HorDynamic(t)   -  RestCoeff(1,1)*s01 -...
    ksi1*2*sqrt((data.body.Mass(1,1) + data.body.AddedMass(1,1))*abs(RestCoeff(1,1)))*v01;
    
    Results.VertTotal(t)    =  Results.VertStatic(t) + ...
    Results.VertDynamic(t)  - data.body.Mass(3,3)*9.81 - ...
    ksi3*2*sqrt((data.body.Mass(3,3) + data.body.AddedMass(3,3))*abs(RestCoeff(3,3)))*v03;
    
    Results.MomTotal(t)     =  Results.MomDynamic(t)  + ... 
    RestCoeff(5,5)*s05  - ... 
    ksi5*2*sqrt((data.body.Mass(5,5) + data.body.AddedMass(5,5))*abs(RestCoeff(5,5)))*v05;
    

    %% Acceleration calculation + RestCoeff(5,5)*s05
        
    a1 = (Results.HorTotal(t))/(data.body.Mass(1,1)+data.body.AddedMass(1,1));
    
    a3 = (Results.VertTotal(t))/(data.body.Mass(3,3)+data.body.AddedMass(3,3));
    
    a5 = (Results.MomTotal(t))/(data.body.Mass(5,5)+data.body.AddedMass(5,5));
    
    
    %% Velocity calculation
    
    v1      =    v01 + (1-gamma)*dt*a01 + gamma*dt*a1;
    
    v3      =    v03 + (1-gamma)*dt*a03 + gamma*dt*a3;

    v5      =    v05 + (1-gamma)*dt*a05 + gamma*dt*a5;
    
    %% Displacement calculation
    
    s1      =    s01 + dt*v01 + (0.5 - beta)*dt^2*a01 + beta*dt^2*a1;
    
    s3      =    s03 + dt*v03 + (0.5 - beta)*dt^2*a03 + beta*dt^2*a3;
    
    s5      =    s05 + dt*v05 + (0.5 - beta)*dt^2*a05 + beta*dt^2*a5;
    
    %% Next displacement step 

    ds1     =    s1 - s01;
    ds3     =    s3 - s03;
    ds5     =    s5 - s05;
    
    dv5     =    v5 - v05;
    %% Resetting initial values for next step 
    
    a01     =    a1;
    a03     =    a3;
    a05     =    a5;
    
    v01     =    v1;
    v03     =    v3;
    v05     =    v5;
    
    s01     =    s1;
    s03     =    s3;
    s05     =    s5;
     
    %% Storing variables to results 
    
    Results.a1(t)   =   a1;
    Results.a3(t)   =   a3;
    Results.a5(t)   =   a5;
    
    Results.v1(t)   =   v1;
    Results.v3(t)   =   v3;
    Results.v5(t)   =   v5;
    
    Results.s1(t)   =   s1;
    Results.s3(t)   =   s3;
    Results.s5(t)   =   ds5*180/pi;  % Pitch results in degrees
    
    %% Moves the structure according to calculated displacements
    
    data.body.xmid  =   data.body.xmid + ds1;
     
    data.body.zmid  =   data.body.zmid + ds3;
   
    
    %% Pitch displacements are calculated independently of displacements in
    % heave and surge
    
    analysis.x1  =   data.body.xmid + ds1;
    
    analysis.z1  =   data.body.zmid + ds3;
    
    for i=1:size(analysis.z1,1)
        for j=1:size(analysis.z1,2)
            
            ds5x = analysis.z1(i,j)*tan(ds5);
            
            ds5z = analysis.x1(i,j)*tan(ds5);
            
            analysis.x1(i,j)  =   analysis.x1(i,j) + ds5x;

            analysis.z1(i,j)  =   analysis.z1(i,j) + ds5z;
            
        end
    end
    
   
end
