%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              RestMooring.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Description: This function calculates the restoring coefficients due to
%mooring lines on the structure. They are necessary for proper surge
%results
%
%Called by: Restoring
%
%Calls: Vol, deltaVCalc, GZcalc
%
%Outputs: C, Fr
%
%Version: 1.0
%Written by: Nils Gunnar Viko and Einar B. Glomnes
%
%Version: 1.0
%Written by: Jon Marius Aasheim, 2012
%Verifed, added comments and restructuring of the code.
%This part of the restoring calculations works well, can be kept as-is
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function varargout = RestMooring(data,rnow,call)

d  = data.body.d;
KG = data.body.KG;

%%There are two different calls in this function, 1 calculates the
%%restoring coefficient and the other calculates the force

if call == 1
    C = sparse(zeros(6));
    
    if isfield(data,'mooring')
        h   = data.env.h;

        for i = 1:length(data.mooring)
            
            %% Creates local variables for readability
            TH0     =   data.mooring(i).TH0;
            w       =   data.mooring(i).w;
            l       =   data.mooring(i).l;
            psi     =   data.mooring(i).psi;
            zi      =   data.mooring(i).zi;
            xi      =   data.mooring(i).xi;
            
            %%Calculates the initial X position
            X0      =   l -(h+zi)*(1+2*TH0/(h+zi)/w)^(1/2) + ...
                        TH0/w * acosh(1+(h+zi)*w/TH0);
            
            %%Calculates the actual x position
            Xreal   =   X0 - rnow(1,1)*cos(psi);
            
            %% Sets iteration values
            THEst   =   TH0;
            XEst    =   X0;
            
            %%calculates the force step
            df      =   1/w * ( (-2/ (1+2*(THEst/w)/(h+zi))^(1/2)) ...
                        + acosh(1+(h+zi)/(THEst/w)));

            error   =   abs(XEst - Xreal);
            
            %%iterates to find an acceptable value within the error
            %%tolerance
            n=1;
            while error > 0.05;

                XEst    =   l -(h+zi)*(1+2*THEst/(h+zi)/w)^(1/2) + ...
                            THEst/w * acosh(1+(h+zi)*w/THEst);

                f       =   XEst -Xreal;
                
                df      =   1/w * ( (-2/ (1+2*(THEst/w)/(h+zi))^(1/2)) ...
                            + acosh(1+(h+zi)/(THEst/w)));

                if df == 0;
                    break
                end


                THEst   =   max(THEst - f/df,1);

                if n == 1000
                   
                    break
                
                end
                error   =   abs(f);
                
                n = n+1;
            end
            
            TV      =    w*(h+zi-cos(psi)*xi*sin(rnow(5,1)))*sqrt(1+2*...
                         abs(THEst)/w/(h+zi));
            
            %%f(x)
            
            fx      =    - THEst*cos(psi)*(d-KG+zi*sin(rnow(5,1))) ...
                         + TV*xi*cos(rnow(5,1));
            
            %% f(x+h)
            
            fxh     =    - THEst*cos(psi)*(d-KG+zi*sin(rnow(5,1)+0.01))+...
                         TV*xi*cos(rnow(5,1)+0.01);
            
            %% finding the restoring coefficients
            C(5,5)  =    C(5,5) + (fxh - fx)/0.01;
            C(1,1)  =    C(1,1) + df^-1;

        end
    end
    
    varargout{1} = C;
    
elseif call == 2
    
    Fr      =       sparse(zeros(6,1));
    
    if isfield(data,'mooring')
        
        h   =   data.env.h;

        for i = 1:length(data.mooring)
            
            %% Creates local variables for readability
            TH0     =   data.mooring(i).TH0;
            w       =   data.mooring(i).w;
            l       =   data.mooring(i).l;
            psi     =   data.mooring(i).psi;
            zi      =   data.mooring(i).zi;
            xi      =   data.mooring(i).xi;
            
            %%Calculates the initial X position
            X0      =   l -(h+zi)*(1+2*TH0/(h+zi)/w)^(1/2) + ...
                        TH0/w * acosh(1+(h+zi)*w/TH0);
            
            %%Calculates the actual x position
            Xreal   =   X0 - rnow(1,1)*cos(psi);
            %% Sets iteration values
            THEst   =   TH0;
            XEst    =   X0;

            error   =   abs(XEst - Xreal);
            
            %%iterates to find an acceptable value within the error
            %%tolerance
            n=1;
            while error > 0.05;

                XEst    =   l -(h+zi)*(1+2*THEst/(h+zi)/w)^(1/2) + ...
                            THEst/w * acosh(1+(h+zi)*w/THEst);

                f       =   XEst -Xreal;
                
                df      =   1/w * ( (-2/ (1+2*(THEst/w)/(h+zi))^(1/2)) ...
                            + acosh(1+(h+zi)/(THEst/w)));

                if df == 0;
                    break
                end


                THEst   =   max(THEst - f/df,1);

                if n== 1000
                   break
                end
                
                error   =   abs(f);
                
                n=n+1;
            end
            
            %%Vertical tension
            TV      =       w*(h+zi-cos(psi)*xi*sin(rnow(5,1)))*...
                            sqrt(1+2*abs(THEst)/w/(h+zi));
            
            %%Finding the restoring forces            
            Fr(1,1) =       Fr(1,1) + THEst*cos(psi);
            Fr(5,1) =       Fr(5,1) - THEst*cos(psi)*...
                            (d-KG+zi*sin(rnow(5,1))) + TV*xi*cos(rnow(5,1));
        end
    end
    
    varargout{1} = Fr;
end
end